//
//  NotesTests.swift
//  SlipboxProjectTests
//
//  Created by Karin Prater on 23.11.22.
//

import XCTest
@testable import SlipboxProject
import CoreData

final class NotesTests: XCTestCase {
    
    var controller: PersistenceController!
    
    var context: NSManagedObjectContext {
        controller.container.viewContext
    }

    override func setUpWithError() throws {
        self.controller = PersistenceController.createEmpty()
    }

    override func tearDownWithError() throws {
        self.controller = nil
    }

    func test_Notes_Convenience_Init() {
        
        let noteTitle = "new"
        let note = Note(title: noteTitle, context: context)
        
        XCTAssertTrue(note.title == noteTitle, "Note should have the title given in the convenience initalizer")
    }
    
    func test_Notes_CreationDate() {
        
        let note = Note(context: context)
       
        let noteConvenient = Note(title: "new", context: context)
        
        XCTAssertNotNil(note.creationDate, "notes should have creationDate property")
        XCTAssertNotNil(noteConvenient.creationDate)
        //XCTAssertTrue(noteConvenient.creationDate != nil)
        //XCTAssertFalse(noteConvenient.creationDate == nil)
    }
    
    func test_Notes_Updating_Title() {
        
        let note = Note(title: "old", context: context)
        note.title = "new"
        
        XCTAssertTrue(note.title == "new")
    }
    
    
    func test_Fetch_All_Predicate() {
        
        _ = Note(title: "default note", context: context)
        let fetch = Note.fetch(NSPredicate.all)
        
        let fetchedNotes = try? context.fetch(fetch)
        
        XCTAssertNotNil(fetchedNotes)
        XCTAssertTrue(fetchedNotes!.count > 0, "Predicate of all should fetch at least 1 object")
        
    }
    
    func test_Fetch_None_Predicate() {
        
        _ = Note(title: "default note", context: context)
        let fetch = Note.fetch(NSPredicate.none)
        
        let fetchedNotes = try? context.fetch(fetch)
        
        XCTAssertNotNil(fetchedNotes)
        XCTAssertTrue(fetchedNotes!.count == 0, "Predicate of none should not fetch any objects")
        
    }
    
    func test_Delete_Note() {
        
        let note = Note(title: "default note", context: context)
        
        Note.delete(note: note)
        
        let fetchedNotes = try? context.fetch(Note.fetch(.all))
        
        XCTAssertTrue(fetchedNotes?.count == 0, "deleted note should not be in data base")
        XCTAssertFalse(fetchedNotes!.contains(note))
        
    }
    
    func test_Asynchronous_Save() {
        
        expectation(forNotification: .NSManagedObjectContextDidSave, object: context) { _ in
            return true
        }
        
        let note = Note(title: "default note", context: context)
        controller.save()
        
        waitForExpectations(timeout: 2.0) { error in
            XCTAssertNil(error, "saving did not complete")
        }
    }
    
    
}
